"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  SpotifyClient: () => SpotifyClient
});
module.exports = __toCommonJS(src_exports);
var import_node_fetch = __toESM(require("node-fetch"), 1);
var import_node_querystring = require("querystring");

// src/utils.ts
var filterTrack = (track) => {
  var _a;
  return {
    title: track == null ? void 0 : track.name,
    artist: (_a = track == null ? void 0 : track.artists) == null ? void 0 : _a.map((artist) => artist == null ? void 0 : artist.name).join(", "),
    link: track == null ? void 0 : track.external_urls.spotify
  };
};
var encodeToBase64 = (str) => Buffer.from(str).toString("base64");

// src/index.ts
var ENDPOINTS = {
  accessToken: "https://accounts.spotify.com/api/token",
  currentlyPlaying: "https://api.spotify.com/v1/me/player/currently-playing",
  lastPlayed: "https://api.spotify.com/v1/me/player/recently-played",
  topItems: "https://api.spotify.com/v1/me/top"
};
var SpotifyClient = class {
  clientId;
  clientSecret;
  refreshToken;
  accessToken = null;
  constructor({ clientId, clientSecret, refreshToken }) {
    this.clientId = clientId;
    this.clientSecret = clientSecret;
    this.refreshToken = refreshToken;
  }
  _genAccesToken = async () => {
    const basicToken = encodeToBase64(`${this.clientId}:${this.clientSecret}`);
    const response = await (0, import_node_fetch.default)(ENDPOINTS.accessToken, {
      method: "POST",
      headers: {
        Authorization: `Basic ${basicToken}`,
        "Content-Type": "application/x-www-form-urlencoded"
      },
      body: (0, import_node_querystring.stringify)({
        grant_type: "refresh_token",
        refresh_token: this.refreshToken
      })
    });
    if (!response.ok) {
      const errorData = await response.json();
      let errorMsg = `API Error: "${errorData == null ? void 0 : errorData.error} - ${errorData == null ? void 0 : errorData.error_description}"`;
      if ((errorData == null ? void 0 : errorData.error_description) == "Refresh token revoked") {
        errorMsg += "\n\u{1F4A1}Tip: Generate a new refresh token with https://github.com/rocktimsaikia/spotify-rtoken-cli\n\n";
      }
      throw errorMsg;
    }
    const responseData = await response.json();
    return responseData == null ? void 0 : responseData.access_token;
  };
  getCurrentTrack = async ({
    fallbackToLastPlayed = true
  } = {}) => {
    try {
      if (this.accessToken === null) {
        this.accessToken = await this._genAccesToken();
      }
      const headers = { Authorization: `Bearer ${this.accessToken}` };
      const response = await (0, import_node_fetch.default)(ENDPOINTS.currentlyPlaying, { headers });
      if (response.status === 401) {
        this.accessToken = await this._genAccesToken();
        return this.getCurrentTrack({ fallbackToLastPlayed });
      }
      let isPlaying = true;
      if (response.status === 204) {
        isPlaying = false;
        return fallbackToLastPlayed ? { isPlaying, ...(await this.getRecentTracks())[0] } : null;
      }
      const responseData = await response.json();
      const currentTrack = filterTrack(responseData.item);
      return { isPlaying, ...currentTrack };
    } catch (error) {
      throw new Error(error);
    }
  };
  getRecentTracks = async (limit = 1) => {
    try {
      if (this.accessToken === null)
        this.accessToken = await this._genAccesToken();
      if (limit > 50 || limit < 1)
        throw new Error("Limit must be between 1 and 50");
      const headers = { Authorization: `Bearer ${this.accessToken}` };
      const response = await (0, import_node_fetch.default)(`${ENDPOINTS.lastPlayed}?limit=${limit}`, { headers });
      if (response.status === 401) {
        this.accessToken = await this._genAccesToken();
        return this.getRecentTracks(limit);
      }
      const responseData = await response.json();
      const lastPlayedTrack = responseData.items.map(({ track }) => filterTrack(track));
      return lastPlayedTrack;
    } catch (error) {
      throw new Error(error);
    }
  };
  getTopTracks = async ({
    limit = 10,
    timeRange = "short"
  } = {}) => {
    try {
      if (this.accessToken === null)
        this.accessToken = await this._genAccesToken();
      if (limit > 50 || limit < 1)
        throw new Error("Limit must be between 1 and 50");
      const headers = { Authorization: `Bearer ${this.accessToken}` };
      const params = (0, import_node_querystring.stringify)({ time_range: `${timeRange}_term`, limit });
      const response = await (0, import_node_fetch.default)(`${ENDPOINTS.topItems}/tracks/?${params}`, { headers });
      if (response.status === 401) {
        this.accessToken = await this._genAccesToken();
        return this.getTopTracks({ limit, timeRange });
      }
      const responseData = await response.json();
      const topTracks = responseData.items.map((item) => ({
        title: item.name,
        artist: item.artists[0].name,
        link: item.external_urls.spotify
      }));
      return topTracks;
    } catch (error) {
      throw new Error(error);
    }
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  SpotifyClient
});
